# lib-prefix.m4 serial 14
dnl Copyright (C) 2001-2005, 2008-2019 Free Software Foundation, Inc.
dnl This file is free software; the Free Software Foundation
dnl gives unlimited permission to copy and/or distribute it,
dnl with or without modifications, as long as this notice is preserved.

dnl From Bruno Haible.
dnl Trimmed to only AC_LIB_PREPARE_MULTILIB function by Brian Lindholm.

dnl AC_LIB_PREPARE_MULTILIB creates
dnl - a variable acl_libdirstem, containing the basename of the libdir, either
dnl   "lib" or "lib64" or "lib/64",
dnl - a variable acl_libdirstem2, as a secondary possible value for
dnl   acl_libdirstem, either the same as acl_libdirstem or "lib/sparcv9" or
dnl   "lib/amd64".
AC_DEFUN([AC_LIB_PREPARE_MULTILIB],
[
  dnl There is no formal standard regarding lib and lib64.
  dnl On glibc systems, the current practice is that on a system supporting
  dnl 32-bit and 64-bit instruction sets or ABIs, 64-bit libraries go under
  dnl $prefix/lib64 and 32-bit libraries go under $prefix/lib. We determine
  dnl the compiler's default mode by looking at the compiler's library search
  dnl path. If at least one of its elements ends in /lib64 or points to a
  dnl directory whose absolute pathname ends in /lib64, we assume a 64-bit ABI.
  dnl Otherwise we use the default, namely "lib".
  dnl On Solaris systems, the current practice is that on a system supporting
  dnl 32-bit and 64-bit instruction sets or ABIs, 64-bit libraries go under
  dnl $prefix/lib/64 (which is a symlink to either $prefix/lib/sparcv9 or
  dnl $prefix/lib/amd64) and 32-bit libraries go under $prefix/lib.
  AC_REQUIRE([AC_CANONICAL_HOST])
  AC_REQUIRE([gl_HOST_CPU_C_ABI_32BIT])

  case "$host_os" in
    solaris*)
      AC_CACHE_CHECK([for 64-bit host], [gl_cv_solaris_64bit],
        [AC_COMPILE_IFELSE(
           [AC_LANG_SOURCE(
              [[#ifdef _LP64
                 int ok;
                #else
                 error fail
                #endif
              ]])],
           [gl_cv_solaris_64bit=yes],
           [gl_cv_solaris_64bit=no])
        ]);;
  esac

  dnl Allow the user to override the result by setting acl_cv_libdirstems.
  AC_CACHE_CHECK([for the common suffixes of directories in the library search path],
    [acl_cv_libdirstems],
    [acl_libdirstem=lib
     acl_libdirstem2=
     case "$host_os" in
       solaris*)
         dnl See Solaris 10 Software Developer Collection > Solaris 64-bit Developer's Guide > The Development Environment
         dnl <https://docs.oracle.com/cd/E19253-01/816-5138/dev-env/index.html>.
         dnl "Portable Makefiles should refer to any library directories using the 64 symbolic link."
         dnl But we want to recognize the sparcv9 or amd64 subdirectory also if the
         dnl symlink is missing, so we set acl_libdirstem2 too.
         if test $gl_cv_solaris_64bit = yes; then
           acl_libdirstem=lib/64
           case "$host_cpu" in
             sparc*)        acl_libdirstem2=lib/sparcv9 ;;
             i*86 | x86_64) acl_libdirstem2=lib/amd64 ;;
           esac
         fi
         ;;
       *)
         dnl If $CC generates code for a 32-bit ABI, the libraries are
         dnl surely under $prefix/lib, not $prefix/lib64.
         if test "$HOST_CPU_C_ABI_32BIT" != yes; then
           dnl The result is a property of the system. However, non-system
           dnl compilers sometimes have odd library search paths. Therefore
           dnl prefer asking /usr/bin/gcc, if available, rather than $CC.
           searchpath=`(if test -f /usr/bin/gcc \
                           && LC_ALL=C /usr/bin/gcc -print-search-dirs >/dev/null 2>/dev/null; then \
                          LC_ALL=C /usr/bin/gcc -print-search-dirs; \
                        else \
                          LC_ALL=C $CC -print-search-dirs; \
                        fi) 2>/dev/null \
                       | sed -n -e 's,^libraries: ,,p' | sed -e 's,^=,,'`
           if test -n "$searchpath"; then
             acl_save_IFS="${IFS= 	}"; IFS=":"
             for searchdir in $searchpath; do
               if test -d "$searchdir"; then
                 case "$searchdir" in
                   */lib64/ | */lib64 ) acl_libdirstem=lib64 ;;
                   */../ | */.. )
                     # Better ignore directories of this form. They are misleading.
                     ;;
                   *) searchdir=`cd "$searchdir" && pwd`
                      case "$searchdir" in
                        */lib64 ) acl_libdirstem=lib64 ;;
                      esac ;;
                 esac
               fi
             done
             IFS="$acl_save_IFS"
           fi
         fi
         ;;
     esac
     test -n "$acl_libdirstem2" || acl_libdirstem2="$acl_libdirstem"
     acl_cv_libdirstems="$acl_libdirstem,$acl_libdirstem2"
    ])
  # Decompose acl_cv_libdirstems into acl_libdirstem and acl_libdirstem2.
  acl_libdirstem=`echo "$acl_cv_libdirstems" | sed -e 's/,.*//'`
  acl_libdirstem2=`echo "$acl_cv_libdirstems" | sed -e '/,/s/.*,//'`
])
